package com.sun.org.apache.bcel.internal.generic;

import com.sun.org.apache.bcel.internal.classfile.RuntimeInvisibleParameterAnnotations;
import com.sun.org.apache.bcel.internal.classfile.RuntimeVisibleParameterAnnotations;
import com.sun.org.apache.bcel.internal.classfile.RuntimeInvisibleAnnotations;
import com.sun.org.apache.bcel.internal.classfile.RuntimeVisibleAnnotations;
import java.io.DataInputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import com.sun.org.apache.bcel.internal.classfile.Attribute;
import com.sun.org.apache.bcel.internal.classfile.ConstantUtf8;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.DataInput;
import java.util.Iterator;
import java.util.ArrayList;
import com.sun.org.apache.bcel.internal.classfile.ElementValuePair;
import com.sun.org.apache.bcel.internal.classfile.AnnotationEntry;
import java.util.List;

public class AnnotationEntryGen
{
    private int typeIndex;
    private List<ElementValuePairGen> evs;
    private final ConstantPoolGen cpool;
    private boolean isRuntimeVisible;
    
    public AnnotationEntryGen(AnnotationEntry a, ConstantPoolGen cpool, boolean copyPoolEntries) {
        this.isRuntimeVisible = false;
        this.cpool = cpool;
        if (copyPoolEntries) {
            this.typeIndex = cpool.addUtf8(a.getAnnotationType());
        }
        else {
            this.typeIndex = a.getAnnotationTypeIndex();
        }
        this.isRuntimeVisible = a.isRuntimeVisible();
        this.evs = this.copyValues(a.getElementValuePairs(), cpool, copyPoolEntries);
    }
    
    private List<ElementValuePairGen> copyValues(ElementValuePair[] in, ConstantPoolGen cpool, boolean copyPoolEntries) {
        List<ElementValuePairGen> out = new ArrayList();
        for (ElementValuePair nvp : in) {
            out.add(new ElementValuePairGen(nvp, cpool, copyPoolEntries));
        }
        return out;
    }
    
    private AnnotationEntryGen(ConstantPoolGen cpool) {
        this.isRuntimeVisible = false;
        this.cpool = cpool;
    }
    
    public AnnotationEntry getAnnotation() {
        AnnotationEntry a = new AnnotationEntry(this.typeIndex, this.cpool.getConstantPool(), this.isRuntimeVisible);
        for (ElementValuePairGen element : this.evs) {
            a.addElementNameValuePair(element.getElementNameValuePair());
        }
        return a;
    }
    
    public AnnotationEntryGen(ObjectType type, List<ElementValuePairGen> elements, boolean vis, ConstantPoolGen cpool) {
        this.isRuntimeVisible = false;
        this.cpool = cpool;
        this.typeIndex = cpool.addUtf8(type.getSignature());
        this.evs = elements;
        this.isRuntimeVisible = vis;
    }
    
    public static AnnotationEntryGen read(DataInput dis, ConstantPoolGen cpool, boolean b) throws IOException {
        AnnotationEntryGen a = new AnnotationEntryGen(cpool);
        a.typeIndex = dis.readUnsignedShort();
        for (int elemValuePairCount = dis.readUnsignedShort(), i = 0; i < elemValuePairCount; ++i) {
            int nidx = dis.readUnsignedShort();
            a.addElementNameValuePair(new ElementValuePairGen(nidx, ElementValueGen.readElementValue(dis, cpool), cpool));
        }
        a.isRuntimeVisible(b);
        return a;
    }
    
    public void dump(DataOutputStream dos) throws IOException {
        dos.writeShort(this.typeIndex);
        dos.writeShort(this.evs.size());
        for (ElementValuePairGen envp : this.evs) {
            envp.dump(dos);
        }
    }
    
    public void addElementNameValuePair(ElementValuePairGen evp) {
        if (this.evs == null) {
            this.evs = new ArrayList();
        }
        this.evs.add(evp);
    }
    
    public int getTypeIndex() {
        return this.typeIndex;
    }
    
    public final String getTypeSignature() {
        ConstantUtf8 utf8 = (ConstantUtf8)this.cpool.getConstant(this.typeIndex);
        return utf8.getBytes();
    }
    
    public final String getTypeName() {
        return this.getTypeSignature();
    }
    
    public List<ElementValuePairGen> getValues() {
        return this.evs;
    }
    
    @Override
    public String toString() {
        StringBuilder s = new StringBuilder(32);
        s.append("AnnotationGen:[").append(this.getTypeName()).append(" #").append(this.evs.size()).append(" {");
        for (int i = 0; i < this.evs.size(); ++i) {
            s.append(this.evs.get(i));
            if (i + 1 < this.evs.size()) {
                s.append(",");
            }
        }
        s.append("}]");
        return s.toString();
    }
    
    public String toShortString() {
        StringBuilder s = new StringBuilder();
        s.append("@").append(this.getTypeName()).append("(");
        for (int i = 0; i < this.evs.size(); ++i) {
            s.append(this.evs.get(i));
            if (i + 1 < this.evs.size()) {
                s.append(",");
            }
        }
        s.append(")");
        return s.toString();
    }
    
    private void isRuntimeVisible(boolean b) {
        this.isRuntimeVisible = b;
    }
    
    public boolean isRuntimeVisible() {
        return this.isRuntimeVisible;
    }
    
    static Attribute[] getAnnotationAttributes(ConstantPoolGen cp, AnnotationEntryGen[] annotationEntryGens) {
        if (annotationEntryGens.length == 0) {
            return new Attribute[0];
        }
        try {
            int countVisible = 0;
            int countInvisible = 0;
            for (AnnotationEntryGen a : annotationEntryGens) {
                if (a.isRuntimeVisible()) {
                    ++countVisible;
                }
                else {
                    ++countInvisible;
                }
            }
            ByteArrayOutputStream rvaBytes = new ByteArrayOutputStream();
            ByteArrayOutputStream riaBytes = new ByteArrayOutputStream();
            try (final DataOutputStream rvaDos = new DataOutputStream(rvaBytes);
                 final DataOutputStream riaDos = new DataOutputStream(riaBytes)) {
                rvaDos.writeShort(countVisible);
                riaDos.writeShort(countInvisible);
                for (AnnotationEntryGen a2 : annotationEntryGens) {
                    if (a2.isRuntimeVisible()) {
                        a2.dump(rvaDos);
                    }
                    else {
                        a2.dump(riaDos);
                    }
                }
            }
            byte[] rvaData = rvaBytes.toByteArray();
            byte[] riaData = riaBytes.toByteArray();
            int rvaIndex = -1;
            int riaIndex = -1;
            if (rvaData.length > 2) {
                rvaIndex = cp.addUtf8("RuntimeVisibleAnnotations");
            }
            if (riaData.length > 2) {
                riaIndex = cp.addUtf8("RuntimeInvisibleAnnotations");
            }
            List<Attribute> newAttributes = new ArrayList();
            if (rvaData.length > 2) {
                newAttributes.add(new RuntimeVisibleAnnotations(rvaIndex, rvaData.length, new DataInputStream(new ByteArrayInputStream(rvaData)), cp.getConstantPool()));
            }
            if (riaData.length > 2) {
                newAttributes.add(new RuntimeInvisibleAnnotations(riaIndex, riaData.length, new DataInputStream(new ByteArrayInputStream(riaData)), cp.getConstantPool()));
            }
            return (Attribute[])newAttributes.toArray(new Attribute[newAttributes.size()]);
        }
        catch (IOException e) {
            System.err.println("IOException whilst processing annotations");
            e.printStackTrace();
            return null;
        }
    }
    
    static Attribute[] getParameterAnnotationAttributes(ConstantPoolGen cp, List<AnnotationEntryGen>[] vec) {
        int[] visCount = new int[vec.length];
        int totalVisCount = 0;
        int[] invisCount = new int[vec.length];
        int totalInvisCount = 0;
        try {
            for (int i = 0; i < vec.length; ++i) {
                if (vec[i] != null) {
                    for (AnnotationEntryGen element : vec[i]) {
                        if (element.isRuntimeVisible()) {
                            int n2 = i;
                            ++visCount[n2];
                            ++totalVisCount;
                        }
                        else {
                            int n = i;
                            ++invisCount[n];
                            ++totalInvisCount;
                        }
                    }
                }
            }
            ByteArrayOutputStream rvaBytes = new ByteArrayOutputStream();
            try (final DataOutputStream rvaDos = new DataOutputStream(rvaBytes)) {
                rvaDos.writeByte(vec.length);
                for (int j = 0; j < vec.length; ++j) {
                    rvaDos.writeShort(visCount[j]);
                    if (visCount[j] > 0) {
                        for (AnnotationEntryGen element2 : vec[j]) {
                            if (element2.isRuntimeVisible()) {
                                element2.dump(rvaDos);
                            }
                            else {
                                continue;
                            }
                        }
                    }
                }
            }
            ByteArrayOutputStream riaBytes = new ByteArrayOutputStream();
            try (final DataOutputStream riaDos = new DataOutputStream(riaBytes)) {
                riaDos.writeByte(vec.length);
                for (int k = 0; k < vec.length; ++k) {
                    riaDos.writeShort(invisCount[k]);
                    if (invisCount[k] > 0) {
                        for (AnnotationEntryGen element3 : vec[k]) {
                            if (!element3.isRuntimeVisible()) {
                                element3.dump(riaDos);
                            }
                            else {
                                continue;
                            }
                        }
                    }
                }
            }
            byte[] rvaData = rvaBytes.toByteArray();
            byte[] riaData = riaBytes.toByteArray();
            int rvaIndex = -1;
            int riaIndex = -1;
            if (totalVisCount > 0) {
                rvaIndex = cp.addUtf8("RuntimeVisibleParameterAnnotations");
            }
            if (totalInvisCount > 0) {
                riaIndex = cp.addUtf8("RuntimeInvisibleParameterAnnotations");
            }
            List<Attribute> newAttributes = new ArrayList();
            if (totalVisCount > 0) {
                newAttributes.add(new RuntimeVisibleParameterAnnotations(rvaIndex, rvaData.length, new DataInputStream(new ByteArrayInputStream(rvaData)), cp.getConstantPool()));
            }
            if (totalInvisCount > 0) {
                newAttributes.add(new RuntimeInvisibleParameterAnnotations(riaIndex, riaData.length, new DataInputStream(new ByteArrayInputStream(riaData)), cp.getConstantPool()));
            }
            return (Attribute[])newAttributes.toArray(new Attribute[newAttributes.size()]);
        }
        catch (IOException e) {
            System.err.println("IOException whilst processing parameter annotations." + e.getMessage());
            return null;
        }
    }
}
